import i18n from "i18next";
import en from "./en";
import ar from "./ar";
import store from "../redux";
import { add } from "lodash";


    let allowedLanguagesList = null;
    const allowLang = process.env.REACT_APP_ALLOWED_LANGUAGE;
    if (allowLang) {
      allowedLanguagesList = allowLang.split(',')
    }

    const defaultLang = process.env.REACT_APP_DEFAULT_LANGUAGE;

    const resources = {};

    allowedLanguagesList.forEach((language) => {
      if(!resources[language]){
        resources[language]={
          translation: language=="ar" ? ar : en,
        }
      }
    });


i18n.init({
  resources,
  lng: "ar",
  keySeparator: false,
  interpolation: {
    escapeValue: false,
  },
});

export const getTranslation = (label, isRTL = store.getState().general.isRTL, options = {}) => {
  const currentLang = isRTL ? "ar": "en"
  const lang = resources[currentLang] && currentLang || defaultLang
  return i18n.t(label, { lng: lang, ...options });
};

export default i18n;
